'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @file 安装扩展
 * @author wangyisheng@baidu.com (wangyisheng)
 */

var ora = require('ora');
var shelljs = require('shelljs');
var fs = require('fs-extra');
var path = require('path');
var glob = require('glob');

var config = require('./config');
var utils = require('../utils');
var log = require('../utils/log');

function resolve() {
    var dir = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    return path.resolve(process.cwd(), dir);
}

module.exports = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(options) {
        var extensionName, extension, npmName, dirName, isExtensionsDirExist, matchExtensionConfig, isNetworkConnect, spinner, code, jsonObject, recommendFiles;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        extensionName = options.name;
                        extension = config.EXTENSION_MAP[extensionName];

                        if (extension) {
                            _context.next = 5;
                            break;
                        }

                        log.fatal(`没有找到名为 ${extensionName} 的 Lavas 扩展`);
                        return _context.abrupt('return');

                    case 5:
                        npmName = extension.npmName, dirName = extension.dirName;

                        // 这里检查是否重复安装
                        // 1. 检查extensions目录下是否已有目标文件夹
                        // 2. 检查config/extensions.js是否包含对应插件

                        _context.next = 8;
                        return fs.pathExists(resolve(`extensions/${dirName}`));

                    case 8:
                        isExtensionsDirExist = _context.sent;
                        matchExtensionConfig = require(resolve('config/extensions')).find(function (config) {
                            return config.name === extensionName;
                        });

                        if (!(isExtensionsDirExist || matchExtensionConfig)) {
                            _context.next = 13;
                            break;
                        }

                        log.info(`检测到 Lavas 扩展：${extensionName} 已经安装过了，安装完毕`);
                        return _context.abrupt('return');

                    case 13:
                        _context.next = 15;
                        return utils.isNetworkConnect();

                    case 15:
                        isNetworkConnect = _context.sent;

                        if (isNetworkConnect) {
                            _context.next = 20;
                            break;
                        }

                        log.info('创建工程需要下载云端模版');
                        log.info('请确认您的设备处于网络可访问的环境中');
                        return _context.abrupt('return');

                    case 20:
                        spinner = ora('正在安装扩展...');

                        spinner.start();

                        // npm install package
                        console.log('\n');
                        code = shelljs.exec(`npm install ${npmName} --save`).code;

                        if (!(code !== 0)) {
                            _context.next = 27;
                            break;
                        }

                        log.fatal(`npm install ${npmName} 失败`);
                        return _context.abrupt('return');

                    case 27:
                        _context.next = 29;
                        return fs.readJson(resolve(`node_modules/${npmName}/extensions/${dirName}/recommend.json`));

                    case 29:
                        jsonObject = _context.sent;
                        recommendFiles = jsonObject.list;
                        _context.next = 33;
                        return _promise2.default.all([fs.copy(resolve(`node_modules/${npmName}/extensions`), resolve('extensions')), fs.copy(resolve(`node_modules/${npmName}/recommend`), resolve())]);

                    case 33:

                        spinner.stop();

                        log.info('安装扩展完毕');
                        log.info('为了正确使用扩展，您还需要手动修改下列文件以增加对扩展的引用：');
                        recommendFiles.forEach(function (file) {
                            log.info(`* ${file}`);
                        });
                        console.log();
                        log.info('请注意：标记为 lavas-extension-start/lavas-extension-end 之间的内容为需要修改的内容，' + '修改完毕后 recommend 文件可以删除');

                    case 39:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function (_x2) {
        return _ref.apply(this, arguments);
    };
}();