/**
 * @file cli entry
 * @author mj(zoumiaojiang@gmail.com)
 */

'use strict';

var program = require('commander');
var cp = require('child_process');

var log = require('./utils/log');
var checkUpdate = require('./utils/checkUpdate');
var initCommand = require('./scaffold');
var buildCommand = require('./build');
var serverCommand = require('./server');
// const extensionCommand = require('./extensions');

var version = process.env.VERSION || require('../package.json').version;

if (!process.argv[2]) {
    cp.exec('lavas -h', function (err, stdout, stderr) {
        if (err) {
            throw err;
        }
        console.log(stdout);
    });
} else {
    var argv = process.argv[2];

    if (argv === '-v' || argv === '--version') {
        log.info('version: ', version);
    }
}

// 定义 lavas 命令
program.usage('[commands] [options]').arguments('<cmd> [env]').option('-v, --version', '查看当前版本').action(function (cmd, env) {
    if (env) {
        log.error(`\`lavas ${cmd} '${env}\` 命令不存在`);
    } else {
        log.error('`lavas ' + cmd + '` 命令不存在');
    }

    log.info('请查看 `lavas --help`');
});

initCommand(program);
buildCommand(program);
serverCommand(program);
// extensionCommand(program);

// 检查版本更新
checkUpdate();
program.parse(process.argv);