/**
 * @file 脚手架
 * @author mj(zoumiaojiang@gmail.com)
 */

'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * 导出工程
 *
 * @param  {Object} params 导出工程所需的参数
 * @param  {Object} templateConf 导出的工程的配置信息
 */
var exportProject = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(params, templateConf) {
        var spinner;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        spinner = ora('正在导出工程..');


                        spinner.start();
                        _context.next = 4;
                        return lavasScaffold.render(params, templateConf);

                    case 4:
                        spinner.stop();

                        // 为了打印出的内容美观不紧凑
                        console.log('');
                        log.info('项目已创建成功!');
                        log.info('您可以操作如下命令快速开始开发 Lavas 工程：\n\n' + log.chalk.green('cd ' + params.name + '\n' + 'npm install\n' + 'npm run dev'));

                    case 8:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function exportProject(_x, _x2) {
        return _ref.apply(this, arguments);
    };
}();

/**
 * 初始化 pwa 项目入口
 *
 * @param {Object} conf 初始化配置
 */


function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var inquirer = require('inquirer');
var shelljs = require('shelljs');
// const lavasScaffold = require('../../../lavas-scaffold/dist');
var lavasScaffold = require('lavas-scaffold');
var ora = require('ora');
var path = require('path');
var fs = require('fs-extra');

var formQ = require('./formQuestion');
var log = require('../utils/log');
var utils = require('../utils');

var cwd = process.cwd();module.exports = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(conf) {
        var isNetworkOk, spinner, metaSchema, metaParams, templateConf, schema, params, projectTargetPath, isPathExist, ret;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        _context2.next = 2;
                        return utils.isNetworkConnect();

                    case 2:
                        isNetworkOk = _context2.sent;

                        if (isNetworkOk) {
                            _context2.next = 7;
                            break;
                        }

                        log.info('创建工程需要下载云端模版');
                        log.info('请确认您的设备处于网络可访问的环境中');
                        return _context2.abrupt('return');

                    case 7:
                        if (shelljs.which('git')) {
                            _context2.next = 11;
                            break;
                        }

                        log.info('Lavas 命令行依赖 git 工具');
                        log.info('当前环境下没有检测到 git 命令，请确认是否安装 git');
                        return _context2.abrupt('return');

                    case 11:

                        log.info(`欢迎使用 ${log.chalk.green('Lavas')} 解决方案`);
                        log.info('开始新建一个 Lavas PWA 项目\n');

                        // 整个初始化过程分为 6 步

                        // 第一步：从云端配置获取 Meta 配置，确定将要下载的框架和模板 List
                        spinner = ora('正在拉取云端数据，请稍候...');

                        spinner.start();
                        _context2.next = 17;
                        return lavasScaffold.getMetaSchema();

                    case 17:
                        metaSchema = _context2.sent;

                        spinner.stop();

                        // 第二步：等待用户选择将要下载的框架和模板
                        _context2.next = 21;
                        return formQ(metaSchema);

                    case 21:
                        metaParams = _context2.sent;


                        // 第三步：通过用户选择的框架和模板，下载模板
                        spinner.start();
                        _context2.next = 25;
                        return lavasScaffold.download(metaParams);

                    case 25:
                        templateConf = _context2.sent;

                        spinner.stop();

                        // 第四步：根据下载的模板的 meta.json 获取当前模板所需要用户输入的字段 schema
                        _context2.next = 29;
                        return lavasScaffold.getSchema(templateConf);

                    case 29:
                        schema = _context2.sent;
                        _context2.next = 32;
                        return formQ(schema);

                    case 32:
                        params = _context2.sent;


                        // 第六步：渲染模板，并导出到指定的文件夹(当前文件夹)
                        projectTargetPath = path.resolve(params.dirPath || cwd, params.name);

                        params = (0, _assign2.default)({}, metaParams, params);

                        _context2.next = 37;
                        return fs.pathExists(projectTargetPath);

                    case 37:
                        isPathExist = _context2.sent;

                        if (!isPathExist) {
                            _context2.next = 56;
                            break;
                        }

                        if (!conf.force) {
                            _context2.next = 46;
                            break;
                        }

                        _context2.next = 42;
                        return fs.remove(projectTargetPath);

                    case 42:
                        _context2.next = 44;
                        return exportProject(params, templateConf);

                    case 44:
                        _context2.next = 54;
                        break;

                    case 46:
                        _context2.next = 48;
                        return inquirer.prompt([{
                            'type': 'confirm',
                            'message': '存在同名项目，是否覆盖?',
                            'default': false,
                            'name': 'isForce'
                        }]);

                    case 48:
                        ret = _context2.sent;

                        if (!ret.isForce) {
                            _context2.next = 54;
                            break;
                        }

                        _context2.next = 52;
                        return fs.remove(projectTargetPath);

                    case 52:
                        _context2.next = 54;
                        return exportProject(params, templateConf);

                    case 54:
                        _context2.next = 58;
                        break;

                    case 56:
                        _context2.next = 58;
                        return exportProject(params, templateConf);

                    case 58:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this);
    }));

    return function (_x3) {
        return _ref2.apply(this, arguments);
    };
}();