/**
 * @file 通过 Schema 生成可以让用户输入的 Question Form
 * @author mj(zoumiaojiang@gmail.com)
 */

'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

/**
 * 询问 input 类型的参数
 *
 * @param  {string} key    参数的 key
 * @param  {Object} schema schema 内容
 * @param  {Object} params 当前已有的参数
 * @return {Object}        question 需要的参数
 */
var questionInput = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(key, schema, params) {
        var con, name, userInfo, reg;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        con = schema[key];
                        name = con.name;


                        con.validate = function () {
                            return !!1;
                        };

                        // 如果输入项是 author 或者 email 的，尝试的去 git config 中拿默认的内容

                        if (!(key === 'author' || key === 'email')) {
                            _context.next = 8;
                            break;
                        }

                        _context.next = 6;
                        return getGitInfo();

                    case 6:
                        userInfo = _context.sent;


                        con.default = userInfo[key] || con.default;

                    case 8:
                        if (key === 'dirPath') {
                            con.default = path.resolve(process.cwd(), con.default || '');
                            con.validate = function (value) {
                                var nowPath = path.resolve(process.cwd(), value || '');

                                if (!fs.existsSync(nowPath)) {
                                    return con.invalidate || '输入不符合规范';
                                }
                                return true;
                            };
                        }

                        if (con.regExp) {
                            reg = new RegExp(con.regExp);


                            con.validate = function (value) {
                                if (!reg.test(value)) {
                                    return con.invalidate || '输入不符合规范';
                                }
                                return true;
                            };
                        }

                        return _context.abrupt('return', {
                            'type': con.type === 'password' ? 'password' : 'input',
                            'name': key,
                            'message': `请输入${name}: `,
                            'default': con.default,
                            'validate': con.validate
                        });

                    case 11:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function questionInput(_x, _x2, _x3) {
        return _ref.apply(this, arguments);
    };
}();

/**
 * 解析schme, 生成 form 表单
 *
 * @param  {Object} schema  传入的 schema 规则
 * @return {Object}         获取的 form 参数
 */


function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var inquirer = require('inquirer');
var childProcess = require('child_process');

var path = require('path');
var fs = require('fs-extra');

var log = require('../utils/log');

/**
 * 获取当前用户的 git 账号信息
 *
 * @return {Promise} promise 对象
 */
function getGitInfo() {
    var exec = childProcess.execSync;

    return new _promise2.default(function (resolve, reject) {
        var author = void 0;
        var email = void 0;

        try {
            // 尝试从 git 配置中获取
            author = exec('git config --get user.name');
            email = exec('git config --get user.email');
        } catch (e) {}
        author = author && author.toString().trim();
        email = email && email.toString().trim();
        resolve({ author, email });
    });
}

/**
 * 询问 list 类型的参数 (多选或者单选)
 *
 * @param  {string} key    参数的 key
 * @param  {Object} schema schema 内容
 * @param  {Object} params 当前已有的参数
 * @return {Object}        question 需要的参数
 */
function questionList(key, schema, params) {
    var con = schema[key];
    var sourceList = [];
    var choiceList = [];

    if (!con.dependence) {
        sourceList = con.list;
    } else if (con.depLevel > 0) {

        // 表示是级联的操作
        var dependence = con.dependence;
        var ref = con.ref;
        var depList = schema[dependence].list;
        var depValue = params[dependence] || schema[dependence].list[0].value;

        depList.forEach(function (depItem) {
            if (depItem.value === depValue) {
                sourceList = depItem.subList && depItem.subList[ref] ? depItem.subList[ref] : [];
            }
        });
    }

    sourceList.forEach(function (item) {
        var url = '';
        var desc = log.chalk.gray('\n\n    ' + item.desc);

        if (item.url) {
            url = '\n\n    - ' + log.chalk.yellow.bold.underline(item.url);
        } else if (item.imgs && item.imgs[0]) {
            item.imgs.forEach(function (imgO) {
                var item = '\n\n    - ' + log.chalk.yellow.bold.underline(imgO.src) + (imgO.alt ? ' - ' + imgO.alt : '');
                url += item;
            });
        } else if (item.img) {
            url = '\n\n    - ' + log.chalk.yellow.bold.underline(item.img);
        }

        choiceList.push({
            'value': item.value,
            'name': `${item.name}${desc}${url}`,
            'short': item.value
        });
    });

    return {
        'type': 'list',
        'name': key,
        'message': `选择一个${con.name} (${log.chalk.green('按上下键选择')}): `,
        'choices': choiceList,
        'default': choiceList[0],
        'checked': !!con.checkbox,
        'pageSize': 1000
    };
}

/**
 * 询问 boolean 类型的参数
 *
 * @param  {string} key    参数的 key
 * @param  {Object} schema schema 内容
 * @param  {Object} params 当前已有的参数
 * @return {Object}        question 需要的参数
 */
function questionYesOrNo(key, schema, params) {
    var con = schema[key];

    return {
        'type': 'confirm',
        'name': key,
        'default': false,
        'message': `${con.name}? :`
    };
}module.exports = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(schema) {
        var params, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, key, con, type, opts, data;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        params = {};
                        _iteratorNormalCompletion = true;
                        _didIteratorError = false;
                        _iteratorError = undefined;
                        _context2.prev = 4;
                        _iterator = (0, _getIterator3.default)((0, _keys2.default)(schema));

                    case 6:
                        if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                            _context2.next = 35;
                            break;
                        }

                        key = _step.value;
                        con = schema[key];
                        type = con.type;
                        opts = {};
                        data = {};
                        _context2.t0 = type;
                        _context2.next = _context2.t0 === 'string' ? 15 : _context2.t0 === 'number' ? 15 : _context2.t0 === 'password' ? 15 : _context2.t0 === 'boolean' ? 19 : _context2.t0 === 'list' ? 21 : 23;
                        break;

                    case 15:
                        _context2.next = 17;
                        return questionInput(key, schema, params);

                    case 17:
                        opts = _context2.sent;
                        return _context2.abrupt('break', 23);

                    case 19:
                        opts = questionYesOrNo(key, schema, params);
                        return _context2.abrupt('break', 23);

                    case 21:
                        opts = questionList(key, schema, params);
                        return _context2.abrupt('break', 23);

                    case 23:
                        if (!(type === 'list' && con.list.length === 1)) {
                            _context2.next = 27;
                            break;
                        }

                        data[key] = con.list[0].value;
                        _context2.next = 31;
                        break;

                    case 27:
                        if (con.disable) {
                            _context2.next = 31;
                            break;
                        }

                        _context2.next = 30;
                        return inquirer.prompt([opts]);

                    case 30:
                        data = _context2.sent;

                    case 31:

                        params = (0, _assign2.default)({}, params, data);

                    case 32:
                        _iteratorNormalCompletion = true;
                        _context2.next = 6;
                        break;

                    case 35:
                        _context2.next = 41;
                        break;

                    case 37:
                        _context2.prev = 37;
                        _context2.t1 = _context2['catch'](4);
                        _didIteratorError = true;
                        _iteratorError = _context2.t1;

                    case 41:
                        _context2.prev = 41;
                        _context2.prev = 42;

                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }

                    case 44:
                        _context2.prev = 44;

                        if (!_didIteratorError) {
                            _context2.next = 47;
                            break;
                        }

                        throw _iteratorError;

                    case 47:
                        return _context2.finish(44);

                    case 48:
                        return _context2.finish(41);

                    case 49:
                        return _context2.abrupt('return', params);

                    case 50:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this, [[4, 37, 41, 49], [42,, 44, 48]]);
    }));

    return function (_x4) {
        return _ref2.apply(this, arguments);
    };
}();