/**
 * @file dev server
 * @author mj(zoumiaojiang@gmail.com)
 */

'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var path = require('path');
var fs = require('fs-extra');
var LavasCore = require('lavas-core');
var Koa = require('koa');

var log = require('../utils/log');
var utils = require('../utils');

/**
 * 处理 dev server 逻辑
 *
 * @param {Object} options action 的参数
 */
module.exports = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(options) {
        var rootDir, serverFile, isServerFileExist, app, isProd, port, core;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        rootDir = utils.getLavasProjectRoot();
                        serverFile = path.resolve(rootDir, 'server.js');


                        log.info('正在启动 Lavas 调试服务器...');

                        _context.next = 5;
                        return fs.pathExists(serverFile);

                    case 5:
                        isServerFileExist = _context.sent;

                        if (!isServerFileExist) {
                            _context.next = 10;
                            break;
                        }

                        // 如果是开发者自己指定的 server.js 启动调试服务器的话
                        // lavas dev 的参数都失效，由开发者自己在服务器文件中自行实现逻辑
                        require(serverFile);
                        _context.next = 30;
                        break;

                    case 10:
                        log.warn('Lavas 没有检测到项目根目录下含有 server.js 文件!');

                        // 如果项目根目录下不存在 server.js 文件，就走 Lavas 默认的调试服务器
                        app = new Koa();
                        isProd = options.isProd;
                        port = options.port || 3000;
                        _context.prev = 14;
                        core = new LavasCore(rootDir);

                        if (isProd) {
                            _context.next = 21;
                            break;
                        }

                        _context.next = 19;
                        return core.build('development');

                    case 19:
                        _context.next = 23;
                        break;

                    case 21:
                        _context.next = 23;
                        return core.runAfterBuild('production');

                    case 23:

                        app.use(core.koaMiddleware());
                        app.listen(port, function () {
                            return log.info('已经在 http://localhost:' + port + ' 启动 Lavas 调试服务器');
                        });
                        _context.next = 30;
                        break;

                    case 27:
                        _context.prev = 27;
                        _context.t0 = _context['catch'](14);

                        console.error(_context.t0);

                    case 30:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this, [[14, 27]]);
    }));

    return function (_x) {
        return _ref.apply(this, arguments);
    };
}();