/**
 * @file 工具包
 * @author mj(zoumiaojiang@gmail.com)
 */

'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var os = require('os');
var fs = require('fs-extra');
var path = require('path');
var dns = require('dns');

var CWD = process.cwd();

module.exports = {

    /**
     * 获取项目根目录
     *
     * @return {string} 目录 Path
     */
    getHome() {
        var dir = process.env[os.platform() === 'win32' ? 'APPDATA' : 'HOME'] + path.sep + '.lavas-project';

        // 如果这个目录不存在，则创建这个目录
        !fs.existsSync(dir) && fs.mkdirSync(dir);

        return dir;
    },

    /**
     * 检测当前网络环境
     *
     * @return {boolean} 是否联网
     */
    isNetworkConnect() {
        return new _promise2.default(function (resolve) {
            dns.lookup('baidu.com', function (err) {
                return resolve(!(err && err.code === 'ENOTFOUND'));
            });
        });
    },

    /**
     * 获取 Lavas 项目的根目录
     *
     * @return {string} 项目根目录
     */
    getLavasProjectRoot() {

        var pathList = CWD.split(path.sep);

        while (pathList.length > 1) {
            var currentPath = pathList.join(path.sep);
            var fileName = 'package.json';
            var filePath = path.resolve(currentPath, fileName);

            if (fs.existsSync(filePath)) {
                var packageJsonContent = fs.readFileSync(filePath, 'utf-8');
                var packageJson = JSON.parse(packageJsonContent);

                if (packageJson.lavas) {
                    return pathList.join(path.sep);
                }
            }
            pathList.pop();
        }

        return CWD;
    }
};